import gtk,gobject

# NMR Friend imports
import NMRFriend

class Notebook_page:
  """Abstract class for NMR notebook page"""
  
  def __init__(self,spectrum):
    """Creates instance"""
    self.page_container = None
    self.page_label = None
    self.page_type = None
    self.spectrum = spectrum
  
  def get_container(self):
    """Returns container widget used to build page"""
    return self.page_container
  
  def get_label(self):
    """Returns label widget used to label page"""
    return self.page_label
  
  def build_page(self):
    """Method that builds page content. Must be overridden"""
    pass
  
  def refresh_page(self):
    """Update page display. Must be overridden"""
    pass
  
  def get_page_type(self):
    """Returns page type."""
    return self.page_type

class Notebook_empty_page(Notebook_page):
  """Class used to build an empty page"""
  
  def __init__(self):
    """Creates notebook page containing welcome message"""
    #call parent __init__
    Notebook_page.__init__(self,None)
    
    #build page
    self.build_page()
  
  def build_page(self):
    # Initializes page_container
    self.page_container = gtk.VBox()
    
    self.page_container.show()
    
    self.page_label = gtk.Label("Welcome to NMR Friend!")
    self.page_label.show()
    
    self.label = gtk.Label()
    self.label.set_markup("Select NMR-Friend/Bruker Spectrum file to open in <b><i>File</i></b> Menu.")
    self.label.show()
    
    self.page_container.pack_start(self.label,True)

class Notebook_param_page(Notebook_page):
  """Class used to build parameter notebook page"""
  
  def __init__(self,spectrum,parameters_type = NMRFriend.PARAM_TYPE_MISC):
    """Creates notebook page containing parameter list"""
    #call parent __init__
    Notebook_page.__init__(self,spectrum)
    
    #initialization
    self.param_table = None
    self.dict_param_widget = {}
    
    # Set page type
    if parameters_type == NMRFriend.PARAM_TYPE_ACQU:
      self.page_type = NMRFriend.NOTEBOOK_PAGE_ACQU_PARAM
    elif parameters_type == NMRFriend.PARAM_TYPE_PROC:
      self.page_type = NMRFriend.NOTEBOOK_PAGE_PROC_PARAM
    else:
      self.page_type = NMRFriend.NOTEBOOK_PAGE_MISC_PARAM
    
    self.parameters_tuple = spectrum.get_params_from_type(parameters_type)
    self.parameters_type = parameters_type
    
    
    
    #build page
    self.build_page()
  
  def build_page(self):
    """Build parameter page"""
    # build_page() can be called only when __init__ is called
    if self.page_container is not None:
      return
    
    # Initializes page_container
    self.page_container  =  gtk.ScrolledWindow()
    
    self.page_container.set_border_width(10)
    self.page_container.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    self.page_container.show()
    
    # Create Table to display list
    self.param_table = gtk.Table(len(self.parameters_tuple)+1,3,False)
    self.param_table.set_row_spacings(5)
    self.param_table.set_col_spacings(5)
    self.param_table.set_border_width(10)
    self.param_table.show()
    self.page_container.add_with_viewport(self.param_table)
    
    
    
    # Check if page label is initialized
    
    if self.parameters_type == NMRFriend.PARAM_TYPE_ACQU:
      label = "Acquisition"
    elif self.parameters_type == NMRFriend.PARAM_TYPE_PROC:
      label = "Processing"
    elif self.parameters_type == NMRFriend.PARAM_TYPE_MISC:
      label = "Misc."
    else:
      label = "Something"
    
    self.page_label = gtk.Label(label)
    
    # Create title label
    if self.parameters_type == NMRFriend.PARAM_TYPE_ACQU:
      title_page = "<b><u>Acquisition parameters:</u></b>"
    elif self.parameters_type == NMRFriend.PARAM_TYPE_PROC:
      title_page = "<b><u>Processing parameters:</u></b>"
    elif self.parameters_type == NMRFriend.PARAM_TYPE_MISC:
      title_page = "<b><u>Miscellanous parameters:</u></b>"
    else:
      title_page = "<b><u>Some parameters:</u></b>"
    
    page_title = gtk.Label(title_page)
    page_title.set_alignment(xalign = 0.25,yalign = 0.5)
    page_title.set_use_markup(True)
    page_title.show()
    self.param_table.set_row_spacing(0,30)
    self.param_table.attach(page_title, 0, 3, 0, 1, gtk.FILL | gtk.EXPAND, 0, 0, 0)
    
    # Fill Table rows using parameter tuple
    current_row = 1
    for param in self.parameters_tuple:
      if param.get_visibility():
        # Create Label for name
        name_param = gtk.Label(param.get_name()+":")
        name_param.set_alignment(xalign = 1, yalign = 0.5)
        name_param.show()
        self.param_table.attach(name_param, 0, 1, current_row, current_row+1, gtk.FILL, 0, 0, 0)
      
        # Create Entry for parameter value
        value_param = gtk.Entry()
        value_param.set_text(str(param.get_value()))
        value_param.set_alignment(0)
        value_param.set_width_chars(len(value_param.get_text()))
        value_param.editing_done()
        
        # Connect signals to entry
        value_param.connect("focus-out-event",self.param_lose_focus,param)
        value_param.connect("activate",self.param_activate,param)
        value_param.connect("button-release-event",self.param_click,param)
        
        # Little trick used when parameter should not be editable by user
        value_param.set_property('sensitive',param.get_tweakable())
        value_param.modify_text(gtk.STATE_INSENSITIVE,gtk.gdk.Color(25000,25000,25000))
        value_param.modify_base(gtk.STATE_INSENSITIVE,gtk.gdk.Color(65300,65300,65300))
        
        value_param.show()
        self.param_table.attach(value_param,1,2,current_row,current_row+1,gtk.EXPAND | gtk.FILL, 0, 0, 0)
        
        # Create Label for description
        desc_param = gtk.Label("<i>"+param.get_desc()+"</i>")
        desc_param.set_use_markup(True)
        desc_param.set_alignment(xalign = 0, yalign = 0.5)
        desc_param.show()
        self.param_table.attach(desc_param, 2, 3, current_row, current_row+1, gtk.FILL, 0, 0, 0)
        # Update dictionnary
        self.dict_param_widget[param.get_name()] = (param,value_param)
        
        # Increment row
        current_row += 1
      
  
  def param_lose_focus(self,entry,event,param):
    if entry.get_text() == str(param.get_value()):
      pass
    else:
      entry.set_text(str(param.get_value()))
    
    return False
  
  def param_activate(self,entry,param):
    if util.isNumber(entry.get_text()):
      param.set_value(util.str2num(entry.get_text()))
    
    else:
      entry.set_text(str(param.get_value()))
      print "Error: this was not number dude!"
    
    entry.grab_focus()
    return False
  
  def param_click(self,entry,event,param):
    entry.grab_focus()
    return False
  
  def get_entry_from_value(self,value):
    """Returns entry widget which text value equals value.
    Returns None object if not entry corresponds."""
    value = str(value)
    
    for widget in self.page_container.get_children():
      if hasattr(widget,"get_text"):
        if widget.get_text() == value:
          return widget
    
    return None
  
  def get_entry_from_param(self,param):
    """Returns entry widget which linked parameter equals param.
    Returns None object if not entry is linked to param."""
    entry = None
    if param.get_name() in self.dict_param_widget:
      param,entry = self.dict_param_widget[param.get_name()]
      #print param.get_name(),"is linked to widget:",entry
    
    return entry

class Notebook_moments_page(Notebook_page):
  """Class used to build moments results notebook page"""
  
  def __init__(self,spectrum):
    """Creates notebook page containing parameter list"""
    #call parent __init__
    Notebook_page.__init__(self,spectrum)
    
    #initialization
    self.moments_table = None
    
    # Set page type
    self.page_type = NMRFriend.NOTEBOOK_PAGE_MOMENTS
    
    self.results_tuple = spectrum.moments.get_results_tupled()    
    
    #build page
    self.build_page()
  
  def build_page(self):
    """Build parameter page"""
    # build_page() can be called only when __init__ is called
    if self.page_container is not None:
      return
    
    # Initializes page_container
    self.page_container  =  gtk.ScrolledWindow()
    
    self.page_container.set_border_width(10)
    self.page_container.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    self.page_container.show()
    
    # Create Table to display list
    self.moments_table = gtk.Table(len(self.results_tuple)+1,2,False)
    self.moments_table.set_row_spacings(5)
    self.moments_table.set_col_spacings(5)
    self.moments_table.set_border_width(50)
    self.moments_table.show()
    self.page_container.add_with_viewport(self.moments_table)
    
    
    
    # Check if page label is initialized
    
    self.page_label = gtk.Label("Moments")
    
    # Create title label
    page_title = gtk.Label("<b><u>Moment Calculation Results:</u></b>")
    page_title.set_alignment(xalign = 0.5, yalign = 0.5)
    page_title.set_use_markup(True)
    page_title.show()
    
    self.moments_table.set_row_spacing(0,30)
    self.moments_table.attach(page_title, 0, 2, 0, 1, gtk.FILL, 0, 0, 0)
    
    # Fill Table rows using parameter tuple
    current_row = 1
    for result in self.results_tuple:
      title,value = result
      if title != "title":
        
        # Create Label for name
        name_res = gtk.Label(title+":")
        name_res.set_alignment(xalign = 1, yalign = 0.5)
        name_res.show()
        self.moments_table.attach(name_res, 0, 1, current_row, current_row+1, gtk.FILL, 0, 0, 0)
        # Create Entry for result value
        value_res = gtk.Entry()
        value_res.set_text(value)
        value_res.set_alignment(0)
        value_res.set_width_chars(len(value))
        value_res.editing_done()
        
        # Little trick used when result should not be editable by user
        value_res.set_property('sensitive',False)
        value_res.modify_text(gtk.STATE_INSENSITIVE,gtk.gdk.Color(25000,25000,25000))
        value_res.modify_base(gtk.STATE_INSENSITIVE,gtk.gdk.Color(65300,65300,65300))
        
        value_res.show()
        self.moments_table.attach(value_res,1,2,current_row,current_row+1,yoptions = 0)
      else:
        # Create Label for name
        self.moments_table.set_row_spacing(current_row-1,20)
        name_res = gtk.Label()
        name_res.set_markup("<i><b>"+value+"</b></i>")
        name_res.set_alignment(xalign = 0.5, yalign = 0.5)
        name_res.show()
        self.moments_table.attach(name_res, 0, 2, current_row, current_row+1, gtk.FILL, 0, 0, 0)
      
      # Increment row
      current_row += 1
      
  
  def param_lose_focus(self,entry,event,param):
    if entry.get_text() == str(param.get_value()):
      pass
    else:
      entry.set_text(str(param.get_value()))
    
    return False
  
  def param_activate(self,entry,param):
    if util.isNumber(entry.get_text()):
      param.set_value(util.str2num(entry.get_text()))
    
    else:
      entry.set_text(str(param.get_value()))
      print "Error: this was not number dude!"
    
    entry.grab_focus()
    return False
  
  def param_click(self,entry,event,param):
    entry.grab_focus()
    return False
  
  def get_entry_from_value(self,value):
    """Returns entry widget which text value equals value.
    Returns None object if not entry corresponds."""
    value = str(value)
    
    for widget in self.page_container.get_children():
      if hasattr(widget,"get_text"):
        if widget.get_text() == value:
          return widget
    
    return None
  
  def get_entry_from_param(self,param):
    """Returns entry widget which linked parameter equals param.
    Returns None object if not entry is linked to param."""
    entry = None
    if param.get_name() in self.dict_param_widget:
      param,entry = self.dict_param_widget[param.get_name()]
      #print param.get_name(),"is linked to widget:",entry
    
    return entry
  
class Notebook_spectrum_page(Notebook_page):
  def __init__(self,spectrum):
    """Creates notebook page containing spectrum graph"""
    #call parent __init__
    Notebook_page.__init__(self,spectrum)
    
    # initialization
    self.page_type = NMRFriend.NOTEBOOK_PAGE_SPECTRUM
    self.graph = None
    
    #build page
    self.build_page()
  
  def build_page(self):
    """Build Spectrum page."""
    
    # Builds Label
    self.page_label = gtk.Label("Spectrum")
    
    #Build Container
    self.page_container = gtk.VBox()
    self.page_container.show()
    
    # Build spectrum graph
    self.graph = NMRFriend.NmrGraph.NMR_Graph(self.spectrum,NMRFriend.NMRGRAPH_TYPE_SPC)
    
    self.graph.show()
    
    self.toolbar = NMRFriend.NmrToolbar.Spc_Navigation_Toolbar(self.graph,None)
    
    self.page_container.pack_start(self.toolbar,False,True)
    self.page_container.pack_start(self.graph,True,True)

class Notebook_depaked_page(Notebook_page):
  def __init__(self,spectrum):
    """Creates notebook page containing spectrum graph"""
    #call parent __init__
    Notebook_page.__init__(self,spectrum)
    
    # initialization
    self.page_type = NMRFriend.NOTEBOOK_PAGE_DEPAKED
    self.graph = None
    
    #build page
    self.build_page()
  
  def build_page(self):
    """Build Spectrum page."""
    
    # Builds Label
    self.page_label = gtk.Label("Depaked")
    
    #Build Container
    self.page_container = gtk.VBox()
    self.page_container.show()
    
    # Build spectrum graph
    self.graph = NMRFriend.NmrGraph.NMR_Graph(self.spectrum,NMRFriend.NMRGRAPH_TYPE_DEPAKED)
    
    self.graph.show()
    
    self.toolbar = NMRFriend.NmrToolbar.Depaked_Navigation_Toolbar(self.graph,None)
    
    self.page_container.pack_start(self.toolbar,False,True)
    self.page_container.pack_start(self.graph,True,True)

class Notebook_fid_page(Notebook_page):
  def __init__(self,spectrum):
    """Creates notebook page containing fid graph"""
    #call parent __init__
    Notebook_page.__init__(self,spectrum)
    
    #initialization
    self.graph = None
    self.page_type = NMRFriend.NOTEBOOK_PAGE_FID
    
    
    #build page
    self.build_page()
  
  def build_page(self):
    """Build Spectrum page."""
    
    # Build Label
    self.page_label = gtk.Label("Fid")
    
    # Build Container
    self.page_container = gtk.VBox()
    self.page_container.show()
    
    # Build FID grpah
    self.graph = NMRFriend.NmrGraph.NMR_Graph(self.spectrum,NMRFriend.NMRGRAPH_TYPE_FID)
    
    self.graph.show()
    
    self.toolbar = NMRFriend.NmrToolbar.Fid_Navigation_Toolbar(self.graph,None)
    
    self.page_container.pack_start(self.toolbar,False,True)
    
    self.page_container.pack_start(self.graph,True,True)

class Notebook_title_page(Notebook_page):
  def __init__(self,spectrum):
    """Creates notebook page containing title"""
    #Calls parent __init__
    Notebook_page.__init__(self,spectrum)
    
    #initialization
    self.textview = None
    self.textbuffer = None
    self.scrolledwin = None
    self.toolbar = None
    self.tooltips = None
    self.savebutton = None
    self.revertbutton = None
    
    
    self.title = spectrum.get_title()
    
    # Build page
    self.build_page()
  
  def build_page(self):
    
    # Checks if container is initialized
    if self.page_container == None:
      self.page_container = gtk.VBox()
      self.page_container.show()
    else:
      return
    
    # Build label
    self.page_label = gtk.Label("Title")
    
    # Build text buffer
    self.textbuffer = gtk.TextBuffer()
    self.textbuffer.set_text(self.title)
    
    # Build text view
    self.textview = gtk.TextView(self.textbuffer)
    #self.textview.show()
    
    # Build scrolled window
    self.scrolledwin = gtk.ScrolledWindow()
    self.scrolledwin.set_border_width(10)
    self.scrolledwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    #self.scrolledwin.show()
    
    # Add Text view to scrolled window
    self.scrolledwin.add(self.textview)
    
    # Build toolbar and buttons
    self.toolbar = gtk.Toolbar()
    #self.toolbar.show()
    self.toolbar.set_style(gtk.TOOLBAR_ICONS)
    self.toolbar.set_tooltips(True)
    
    # Creates tootlip
    self.tooltips  =  gtk.Tooltips()
    #self.tooltips.enable()
    
    # Creates Save button
    self.savebutton = gtk.ToolButton(gtk.STOCK_SAVE)
    
    # Add tooltip
    self.savebutton.set_tooltip(self.tooltips,"Save title")
    
    # Creates Revert button
    self.revertbutton = gtk.ToolButton(gtk.STOCK_REVERT_TO_SAVED)
    
    # Add tooltip
    self.revertbutton.set_tooltip(self.tooltips,"Revert to saved title")
    
    # Connect signals
    self.revertbutton.connect("clicked",self.revert_title)
    self.savebutton.connect("clicked",self.save_title)
    
    # Add buttons to toolbar
    self.toolbar.add(self.savebutton)
    self.toolbar.add(self.revertbutton)
    
    #Add button box to vbox
    self.page_container.pack_start(self.toolbar,False,True)
    
    #Add scrolled window to Vbox
    self.page_container.add(self.scrolledwin)
    
    #Little trick to get focus on textview by default
    self.page_container.set_focus_chain((self.textview,))
    
    self.page_container.show_all()
  
  def revert_title(self,widget = None):
    """Revert title to saved"""
    self.textbuffer.set_text(self.spectrum.get_title())
  
  def save_title(self,widget = None):
    """Save text to title"""
    start = self.textbuffer.get_bounds()[0]
    end = self.textbuffer.get_bounds()[1]
    self.spectrum.set_title(self.textbuffer.get_text(start,end))
    
    #trick for nmr set overview
    if hasattr(self.spectrum,"get_canvas"):
      if hasattr(self.spectrum.get_canvas(),"rebuild_figure"):
        self.spectrum.get_canvas().rebuild_figure()
  
  def refresh_page(self):
    self.revert_title()

class NMR_Notebook(gtk.Notebook):
    """Class that handles notebook used to display spectra and infos"""
    def __init__(self):
    
        # Call ancestor __init__
        gtk.Notebook.__init__(self)
        
        # Variable initialization
        self.pages = []

    def add_page(self,page):
        """Add page to notebook"""
        # Add pages to page list
        self.pages.append(page)
        
        # Add page to widget
        self.append_page(page.get_container(),page.get_label())
  
    def empty(self):
        """Remove all page from notebook"""
        # empty page list
        self.pages = []
        
        # Remove children from widget
        for i in range(self.get_n_pages()):
          self.remove_page(0)
    
  
    def get_nth_nmr_page(self,page_num):
        """Returns NMR page or None if no index corresponds"""
        try:
            return self.pages[page_num]
        except IndexError:
            return None
    
  
    def rebuild(self):
        """ Rebuild notebook according to _active_object"""
        # First: empty notebook
        self.empty()
        
        if isinstance(NMRFriend._active_object,NMRFriend.NmrSpectrum.NmrSpc):
            self.build_nmr_spectrum_display()
        else:
            self.build_default_display()
    
  
    def build_nmr_spectrum_display(self):
        """Update notebook to display spectrum and info"""       
        # Recreate each page
        
        #-------------------------------------------------------------
        # First page: spectrum
        #-------------------------------------------------------------
        spectrum_page = Notebook_spectrum_page(NMRFriend._active_object)
        
        #-------------------------------------------------------------
        # Second page: acquisition parameters
        #-------------------------------------------------------------
        acq_page = Notebook_param_page(NMRFriend._active_object,NMRFriend.PARAM_TYPE_ACQU)
        
        
        #-------------------------------------------------------------
        # Third page: Processing parameters
        #-------------------------------------------------------------
        proc_page = Notebook_param_page(NMRFriend._active_object,NMRFriend.PARAM_TYPE_PROC)
        
        #-------------------------------------------------------------
        # Fourth page: Spectrum title
        #-------------------------------------------------------------
        title_page = Notebook_title_page(NMRFriend._active_object)
        
        #-------------------------------------------------------------
        # Fifth page: Fid
        #-------------------------------------------------------------
        fid_page = Notebook_fid_page(NMRFriend._active_object)
        
        #-------------------------------------------------------------
        # Optional Page: Moments Results Page
        #-------------------------------------------------------------
        if NMRFriend._active_object.moments is not None:
            moments_page = Notebook_moments_page(NMRFriend._active_object)
        
        #-------------------------------------------------------------
        # Optional Page: Moments Results Page
        #-------------------------------------------------------------
        if NMRFriend._active_object.depaked is not None:
            depaked_page = Notebook_depaked_page(NMRFriend._active_object)
        
        # Append pages
        self.add_page(spectrum_page)
        self.add_page(acq_page)
        self.add_page(proc_page)
        self.add_page(title_page)
        self.add_page(fid_page)
        
        if NMRFriend._active_object.moments is not None:
            self.add_page(moments_page)
        
        if NMRFriend._active_object.depaked is not None:
            self.add_page(depaked_page)
        
    def build_default_display(self):
        # Add empty page
        page = Notebook_empty_page()
        self.add_page(page)
  
  