import matplotlib,math
from matplotlib.backends.backend_gtk import FigureCanvasGTK
from matplotlib.font_manager import FontProperties
from matplotlib.ticker import FuncFormatter
from matplotlib.ticker import LinearLocator
from matplotlib.axes import Subplot
from matplotlib.transforms import lbwh_to_bbox
from gtk import Table
import gtk

# NMRFRiend imports
import NMRFriend
import NMRFriend.Toolbox.Util as Util


class NMR_GObject:
    """Abstract class used for common features of NMR Graphic objects"""
    def __init__(self,nmr_object):
        self.nmr_object=nmr_object
    
class NMR_Graph(FigureCanvasGTK):
    """Handle NMR spectrum display (Matplotlib stuff inside)!"""
    #Constants definition

    
    def __init__(self,spectrum,gtype=NMRFriend.NMRGRAPH_TYPE_SPC):
        
        self.spectrum=spectrum
        self.graph_type=gtype
        
        self.needs_tb=False
        
        # Create matplotlib Figure (empty graph)
        self.figure = matplotlib.figure.Figure(facecolor="w")
        
        # Execute Parent class init
        FigureCanvasGTK.__init__(self, self.figure) # a gtk.DrawingArea
        
        # Initialize variables
        self.units={}
        self.old_units=None
        
        # Plot variables
        self.plot_x=None
        self.plots_y=[]
        
        # Axes
        self.axes=[]
        
        # Title
        self.title = None
        
        if self.graph_type == NMRFriend.NMRGRAPH_TYPE_FID:
            self._init_fid()
        elif self.graph_type == NMRFriend.NMRGRAPH_TYPE_DEPAKED:
            self._init_depaked()
        else:
            self._init_spc()
        
        self.rescale()
        
        #Connect signals
        self.connect("map-event",self.refresh_title)
    
    def needs_toolbar(self):
        return self.needs_tb
    
    def _init_fid(self):
        self.units[NMRFriend.UNITS_PTS]="Points"
        self.units[NMRFriend.UNITS_SEC]="Seconds"
        
        self.active_units=NMRFriend.UNITS_PTS
        self.plot_x=self.spectrum.get_fid_axis(self.active_units)
        
        fid=self.spectrum.get_fid()
        refid=[]
        imfid=[]
        
        for i in range(0,len(fid)-1,2):
            refid.append(fid[i])
            imfid.append(fid[i+1])
        
        self.plots_y.append(refid)
        self.plots_y.append(imfid)
        
        #draw axes
        self.draw_axes()
    
    
    def _init_spc(self):
        
        self.lims=[]
        self.pivot=None
        self.moments_zero=None
        
        self.units[NMRFriend.UNITS_PTS]="Points"
        self.units[NMRFriend.UNITS_HZ]="Hz"
        self.units[NMRFriend.UNITS_PPM]="PPM"
        
        self.active_units=NMRFriend.UNITS_HZ
        self.plot_x=self.spectrum.get_spc_axis(self.active_units)
        
        self.plots_y.append(self.spectrum.get_real_spc())
        
        # draw axes
        self.draw_axes()
    
    def _init_depaked(self):
        
        self.units[NMRFriend.UNITS_PTS] = "Points"
        self.units[NMRFriend.UNITS_HZ] = "Hz"
        self.units[NMRFriend.UNITS_PPM] = "PPM"
        
        self.active_units = NMRFriend.UNITS_HZ
        self.plot_x = self.spectrum.get_spc_axis(self.active_units)
        
        self.plots_y.append(self.spectrum.get_depaked())
        
        # draw axes
        self.draw_axes()
        
    def get_units(self):
        """Returns active units"""
        return self.units[self.active_units]
    
    def refresh_title(self,*args):
        """Refresh title displayed in spectrum graph"""
        if self.title:
            if self.title.get_text()!=self.spectrum.get_title():
                self.title.set_text(self.spectrum.get_title())
                self.draw()

    def set_x_plot(self,new_plotx):
        """Update plot x and redraw plots"""
        if len(new_plotx)!=len(self.plot_x):
            raise IndexError,"Unconsistant new X axis: "+str(len(new_plotx))+" for "+str(len(self.plot_x))
        else:
            self.plot_x=new_plotx
        
        self.draw_axes()
    
    
    def get_x_lims(self):
        """Return xmin,xmax"""
        return min(self.plot_x),max(self.plot_x)
    
    def get_axis(self):
        """Obsolete!"""
        pass
    
    def get_figure_height(self):
        """Returns figure height."""
        return self.figure.bbox.height()
    
    def get_figure_width(self):
        """Returns figure height."""
        return self.figure.bbox.width()
    
    def get_figure_bounds(self):
        """Returns figure bounds."""
        return self.figure.bbox.get_bounds()
    
    def get_axis_bounds(self):
        """Returns axis bounds."""
        return self.axis.bbox.get_bounds()
    
    def rescale(self):
        """Rescale figure to display whole data."""
        
        self.axes[0].autoscale_view()
        
        self.rescaleX()
    
    def rescaleX(self):
        """Rescale only X axis"""
        if len(self.plot_x)==0:
            return
        
        xmin,xmax=self.plot_x[0],self.plot_x[len(self.plot_x)-1]
        self.axes[0].set_xlim(xmin,xmax)
        self.refresh_figure()
    
    def rescaleY(self):
        """Rescale only Y axis"""
        xmin,xmax=self.axes[0].get_xlim()
        
        self.axes[0].autoscale_view()
        
        self.axes[0].set_xlim(xmin,xmax)
        
        self.refresh_figure()
    
    
    def refresh_figure(self):
        """Redraw the canvases, update the locators."""
        xaxis = getattr(self.axes[0], 'xaxis', None)
        if self.graph_type == NMRFriend.NMRGRAPH_TYPE_SPC or self.graph_type == NMRFriend.NMRGRAPH_TYPE_DEPAKED:
            xaxis.set_major_formatter(FuncFormatter(Util.format_tick))
        yaxis = getattr(self.axes[0], 'yaxis', None)
        locators = []
        if xaxis is not None:
                locators.append(xaxis.get_major_locator())
                locators.append(xaxis.get_minor_locator())                
        if yaxis is not None:
                locators.append(yaxis.get_major_locator())
                locators.append(yaxis.get_minor_locator())                

        for loc in locators:
                loc.refresh()
                
        
        
        # Finally redraws figure
        self.draw_idle()

    def draw_axes(self):
        """Erase axes and redraw all"""
        
        # Store xlims in old units
        if self.old_units!=None:
            old_xmin,old_xmax=self.axes[0].get_xlim()
            new_xmin,new_xmax=self.spectrum.old_units_to_new(self.old_units,self.active_units,(old_xmin,old_xmax))
        
        # Empty figure if not empty
        while len(self.axes)>0:
            self.axes.pop().clear()
        
        
        if len(self.figure.get_axes()) >0:
            self.figure.clear()

        if self.graph_type==NMRFriend.NMRGRAPH_TYPE_FID:
            
            # Setting up axes
            
            if NMRFriend.main_app.options.get_option("fid_unshuffling")==NMRFriend.FID_UNSHUF_NONE:
                self.axes.append(self.figure.add_subplot(1,1,1))
                self.axes[0].set_xlabel(self.units[self.active_units])
                self.axes[0].set_ylabel('Intensity')
                self.axes[0].grid(NMRFriend.main_app.options.get_option("grid"))
                
                # Plot spectrum
                for plotty in self.plots_y:
                    self.axes[0].plot(self.plot_x,plotty,antialiased=False)
                
            
            elif NMRFriend.main_app.options.get_option("fid_unshuffling")==NMRFriend.FID_UNSHUF_HORZ:
                self.axes.append(self.figure.add_subplot(121))
                self.axes.append(self.figure.add_subplot(122,sharex=self.axes[0],sharey=self.axes[0]))
            
            elif NMRFriend.main_app.options.get_option("fid_unshuffling")==NMRFriend.FID_UNSHUF_VERT:
                self.axes.append(self.figure.add_subplot(211))
                self.axes.append(self.figure.add_subplot(212,sharex=self.axes[0],sharey=self.axes[0]))
            
            if NMRFriend.main_app.options.get_option("fid_unshuffling")!=NMRFriend.FID_UNSHUF_NONE:
                
                for axe in self.axes:
                    axe.set_xlabel(self.units[self.active_units])
                    axe.set_ylabel('Intensity')
                    axe.grid(NMRFriend.main_app.options.get_option("grid"))
                
                self.axes[0].plot(self.plot_x,self.plots_y[0],antialiased=False)
                self.axes[1].plot(self.plot_x,self.plots_y[1],antialiased=False)
                
                # remove ticks and ylabel for right subplot
                if NMRFriend.main_app.options.get_option("fid_unshuffling")==NMRFriend.FID_UNSHUF_HORZ:
                    self.axes[1].set_ylabel("")
                
                if NMRFriend.main_app.options.get_option("fid_unshuffling")==NMRFriend.FID_UNSHUF_VERT:
                    self.axes[0].set_xlabel("")
                
                # set titles
                self.axes[0].set_title("Real",weight="bold")
                self.axes[1].set_title("Imaginary",weight="bold")
                
            
            # Add text if empty
            if len(self.plot_x)==0:
                self.axes[0].text(0.5,
                                  0.5,
                                  "No FID available!",
                                  horizontalalignment="center",
                                  verticalalignment="center",
                                  transform=self.axes[0].transAxes,
                                  multialignment="center",
                                  weight="bold",
                                  size=16,
                                  color="r"
                                      )
                
                self.needs_tb=False
            else:
                self.needs_tb=True
        
        elif self.graph_type==NMRFriend.NMRGRAPH_TYPE_DEPAKED:
            # Setting up axes (only one for spectra)
            self.axes.append(self.figure.add_subplot(1,1,1))
            self.axes[0].set_xlabel(self.units[self.active_units])
            self.axes[0].set_ylabel('Intensity')
            self.axes[0].grid(NMRFriend.main_app.options.get_option("grid"))
            self.title=self.axes[0].text(0.01,
                                         0.99,
                                         self.spectrum.get_title(),
                                         horizontalalignment="left",
                                         verticalalignment="top",
                                         transform=self.axes[0].transAxes,
                                         multialignment="left",
                                         size=12,
                                         color="r"
                                         )
            
            # Plot spectrum
            for plotty in self.plots_y:
                self.axes[0].plot(self.plot_x,plotty,antialiased=False)
            
            self.needs_tb=True
            
        else: # Handling spectrum
            
            # Setting up axes (only one for spectra)
            self.axes.append(self.figure.add_subplot(1,1,1))
            self.axes[0].set_xlabel(self.units[self.active_units])
            self.axes[0].set_ylabel('Intensity')
            self.axes[0].grid(NMRFriend.main_app.options.get_option("grid"))
            self.title=self.axes[0].text(0.01,
                                         0.99,
                                         self.spectrum.get_title(),
                                         horizontalalignment="left",
                                         verticalalignment="top",
                                         transform=self.axes[0].transAxes,
                                         multialignment="left",
                                         size=12,
                                         color="r"
                                         )
            
            # Plot spectrum
            for plotty in self.plots_y:
                self.axes[0].plot(self.plot_x,plotty,antialiased=False)
            
            # Add text if empty
            if len(self.plot_x)==0:
                self.axes[0].get_xaxis().set_visible(False)
                self.axes[0].get_yaxis().set_visible(False)
                self.axes[0].text(0.5,
                                  0.5,
                                  "No processed data available!",
                                  horizontalalignment="center",
                                  verticalalignment="center",
                                  transform=self.axes[0].transAxes,
                                  multialignment="center",
                                  weight="bold",
                                  size=16,
                                  color="r"
                                      )
                self.needs_tb=False
            else:
                self.needs_tb=True
            
            # draw moments lims,pivot, and zero if exists
            
            self.draw_moments_lines()
        
        if self.old_units is not None:
            self.axes[0].set_xlim(new_xmin,new_xmax)
            self.refresh_figure()
            self.old_units=None
        else:
            self.rescaleX()
        
    
    def draw_moments_lines(self):
        """Draw lines linkend to moments"""
        
        if self.graph_type == NMRFriend.NMRGRAPH_TYPE_FID:
            return
        
        if self.spectrum.limits is None:
            return
        else:
            # Empty old used
            self.lims=[]
            self.pivot=None
            self.moments_zero=None
            
            # Get points
            llim, rlim = self.spectrum.limits.get_limits()
            pivot = self.spectrum.limits.get_pivot()
            try:
                m1zero = self.spectrum.moments.get_m1_zero()
            except AttributeError:
                m1zero = None
            
            # Draw limits if necessary
            if llim is not None:
                # Translate points to actvie units
                llim_tr=self.spectrum.old_units_to_new(NMRFriend.UNITS_PTS,self.active_units,llim)
        
                rlim_tr=self.spectrum.old_units_to_new(NMRFriend.UNITS_PTS,self.active_units,rlim)
                
                # Draw lines
                self.lims.append(self.axes[0].axvline(llim_tr,ymin=0.0,ymax=0.95,color="#04ED25"))
                self.lims.append(self.axes[0].axvline(rlim_tr,ymin=0.0,ymax=0.95,color="#04ED25"))
        
            
            # Draw pivot if necessary
            if pivot is not None:
                # Translate points to active units
                pivot_tr=self.spectrum.old_units_to_new(NMRFriend.UNITS_PTS,self.active_units,pivot)
                # draw line
                self.pivot=self.axes[0].axvline(x=pivot_tr,ymin=0.0,ymax=0.85,color="#FF5A5A")
                
            
            # Draw M1 zero if necessary
            if m1zero is not None:
                # Translate points to active units
                zero_tr=self.spectrum.old_units_to_new(NMRFriend.UNITS_PTS,self.active_units,m1zero)
                # Draw line
                self.moments_zero=self.axes[0].axvline(zero_tr,ymin=0.9,ymax=0.95,color="m")
    
    def get_active_units(self):
        """Returns active units"""
        return self.active_units
    
    def toggle_units(self):
        """Toggle units in available units and return new active units"""
        self.old_units=self.active_units
        
        keys=self.units.keys()
        for index in range(len(keys)):
            if self.active_units==keys[index]:
                try:
                    self.active_units=keys[index+1]
                except IndexError:
                    self.active_units=keys[0]
                    
                break
        
        # Save units in options
        if self.graph_type==NMRFriend.NMRGRAPH_TYPE_SPC:
            NMRFriend.main_app.options.set_option("spectrum_units",self.active_units)
        elif self.graph_type==NMRFriend.NMRGRAPH_TYPE_FID:
            NMRFriend.main_app.options.set_option("fid_units",self.active_units)
        return self.active_units

def trans_x_bbox(old_bbox,new_xmin,new_xmax):
    old_x0,oldy0,old_width,old_height=old_bbox.get_bounds()
    
    return lbwh_to_bbox(new_xmin,oldy0,new_xmax-new_xmin,old_height)
    