import math

# NMR Friend import
import NMRFriend

class NmrParam:
  """NMR Parameter class"""

  def __init__(self, name, desc="Must be something!", value=None, param_type=NMRFriend.PARAM_TYPE_MISC, visibility=True, user_tweakable=False):
    """Create new NMR param."""
    self.name=name
    self.desc=desc
    self.value=value
    self.param_type=param_type
    self.visibility=visibility
    self.tweakable=user_tweakable

  def get_name(self):
    """Returns parameter name."""
    return self.name
  
  def get_desc(self):
    """Returns description of parameter"""
    return self.desc
  
  def get_tweakable(self):
    """Returns True if parameter can be set by user. Returns False, otherwise"""
    return self.tweakable
  
  def get_value(self):
    """Returns parameter value."""
    return self.value
  
  def set_value(self, newvalue):
    """Sets a new value to parameter."""
    self.value=newvalue
  
  def get_param_type(self):
    """Returns parameter type (acquisition/processing/misc)."""
    return self.param_type
  
  def set_param_type(self, newparam_type):
    """Sets a new value to parameter type.
    Must be one of the following:PARAM_TYPE_ACQU,PARAM_TYPE_PROC,PARAM_TYPE_MISC.
    """
    if newparam_type!=NMRFriend.PARAM_TYPE_ACQU and newparam_type!=NMRFriend.PARAM_TYPE_MISC and newparam_type!=NMRFriend.PARAM_TYPE_PROC:
      raise TypeError
    else:
      self.param_type=newparam_type
      
  def get_visibility(self):
    """Returns parameter visibility in parameter list.
    True if visible, False if not visible
    """
    return self.visibility
  
  def set_visibility(self, newvisibility):
    """Sets a new value to visibility.
    Must be a boolean (True for visible, False for hidden in parameter list)"""
    if type(newvisibility) != type(True):
      raise TypeError
    else:
      self.visibility=newvisibility

class NmrSpc:
  """Defines NMR spectrum."""
  # list containing parameters
  # syntax for parameter:[name,desc,value,param_type,visibility,user_tweakable]
  list_param=[["td", "Time Domain: FID number of points", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
              ["inc_tps", "Time increment between 2 points from FID", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
              ["si", "Spectrum number of points", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["inc_hz", "Increment in Hz between 2 points from spectrum", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["inc_ppm", "Increment in PPM between 2 points from spectrum", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["o1_hz", "Frequency in Hz of spectrum center", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
              ["o1_ppm", "Frequency in PPM of spectrum center", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
              ["sw_hz", "Width of Spectrum in Hz", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
              ["sw_ppm", "Width of Spectrum in PPM", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
              ["sw_p", "Width of processed spectrum in Hz", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["sr_ppm", "Spectrum offset in PPM used for calibration", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["sr_hz", "Spectrum offset in Hz used for calibration", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["phc0", "Zero order phase correction", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["phc1", "First order phase correction", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
              ["pivot", "Pivot point (index: no units) used for phase correction", None, NMRFriend.PARAM_TYPE_PROC, False, False], 
              ]
  
  def __init__(self, fid=[], reSpc=[], imSpc=[]):
    """Create Nmrspc object."""
    
    # Initialization of parameter set
    self.paramset=NmrParamSet(self.__class__.list_param)
    
    # Initialization of phase correction stuff
    self.lastphc0 = None
    self.lastphc1 = None
    
    # Initialization of Spectrum Calculated stuff
    self.limits = None
    self.moments = None
    self.depaked = None
    
    # Initialization of Spectrum title
    self.title=""
    
    # Stock FID & Spectra
    self.fid = fid                              # FID Points
    self.reSpc = reSpc                          # Real part of spectrum Points
    self.imSpc = imSpc                          # Imaginary part of spectrum
    
    # Update parameter set
    self.set_param_value("td", len(self.fid))
    self.set_param_value("si", len(self.reSpc))
    
    # Flag for processing
    self.processed=False
  
  def get_depaked(self):
      return self.depaked
  
  def set_depaked(self, depaked):
      if len(depaked) != len(self.reSpc):
          raise ValueError("Non consistent Depaked spectrum!")
      self.depaked = depaked
      
  def set_title(self, newtitle):
    """Set a new value to Spectrum title."""
    #print "newtitle:",newtitle
    self.title=str(newtitle)
  
  def get_title(self):
    """Get Spectrum title value."""
    return self.title
  
  def set_param_value(self, param_name, newvalue):
    """Get parameter from parameter set and sets it to a new value.
    Returns True if OK, False if not.
    """
    if self.paramset.param_exists(param_name):
      self.paramset.get_param(param_name).set_value(newvalue)
      return True
    else:
      print "Parameter", param_name, "does not exist! Can't set value"
      return False

  def get_param_value(self, param_name):
    """Get parameter from parameter set.
    Returns its value if exists. Otherwise, returns None.
    """
    if self.paramset.param_exists(param_name):
      return self.paramset.get_param(param_name).get_value()
    else:
      print "Parameter", param_name, "does not exist! Can't get value"
      return None

  def set_fid(self, newfid):
    """Updates FID.
    Returns True if OK, False if not.
    """
    self.fid=newfid # Sets new FID
    self.set_param_value("td", len(self.fid)) # Update parameter
    return True
 
  def get_fid(self):
    """Returns FID points"""
    return self.fid
  
  def get_real_spc(self):
    """Returns Real part of Spectrum"""
    return self.reSpc
  
  def get_imag_spc(self):
    """Returns Imaginary part of Spectrum"""
    return self.imSpc
  
  def get_spc_axis(self, units=NMRFriend.UNITS_PTS):
    """Returns axis value for spectrum plot using given units"""
    
    # Check if processed spectrum exists
    if len(self.reSpc)==0:
      return []
    
    if units==NMRFriend.UNITS_PTS:
      return range(1, len(self.reSpc)+1)
    
    if units==NMRFriend.UNITS_PPM:
      axis=[]
      
      # Calculates left limit of spectrum (i.e. positive limits)
      llim= self.get_param_value("o1_ppm")+self.get_param_value("sr_ppm")+self.get_param_value("sw_ppm")/2.0
      inc=self.paramset.get_param("inc_ppm").get_value()
      for i in range (len(self.reSpc)):
          axis.append(llim-i*inc)
      
      #print "O1p:%.3f"%self.get_param_value("o1_ppm")
      #print "SR:%.3f"%self.get_param_value("sr_ppm")
      #print "SW/2:%.3f"%(self.get_param_value("sw_ppm")/2.0)
      #print "llim:%.3f et inc:%.4f"%(llim, inc)
      return axis
    
    if units==NMRFriend.UNITS_HZ:
      axis=[]
      
      # Calculates left limit of spectrum (i.e. positive limits)
      llim= self.get_param_value("o1_hz")+self.get_param_value("sr_hz")+self.get_param_value("sw_hz")/2.0
      
      inc=self.paramset.get_param("inc_hz").get_value()
      for i in range (len(self.reSpc)):
        axis.append(llim-i*inc)
      
      return axis
  
  def get_fid_axis(self, units=NMRFriend.UNITS_PTS):
    """Returns axis for FID plot using given units"""
    
    if units==NMRFriend.UNITS_PTS:
      return range(1, len(self.fid)/2+1)
    
    if units==NMRFriend.UNITS_SEC:
      axis=[]
      for i in range(0, len(self.fid)/2):
        axis.append(float(i) * self.get_param("inc_tps").get_value())
      return axis
    
  def ppm2point(self, ppmval):
    """Converts PPM value to point index.
    Returns point index.
    """
    if self.processed:
      return (self.get_param_value("o1_ppm")+self.get_param_value("sr_ppm")+self.get_param_value("sw_ppm")/2.0-ppmval)/self.get_param_value("inc_ppm")+1
 
  def point2ppm(self, ptval):
    """Converts point index to PPM value.
    Returns PPM value.
    """
    if self.processed:
      llim= self.get_param_value("o1_ppm")+self.get_param_value("sr_ppm")+self.get_param_value("sw_ppm")/2.0
      return llim-(ptval-1)*self.get_param_value("inc_ppm")
  
  def hz2point(self, hzval):
    """Converts Hz value to point index.
    Returns point index.
    """
    if self.processed:
      return (self.get_param_value("o1_hz")+self.get_param_value("sr_hz")+self.get_param_value("sw_hz")/2.0-hzval)/self.get_param_value("inc_hz")+1
  
  def point2hz(self, ptval):
    """Converts point index to Hz value.
    Returns Hz value.
    """
    if self.processed:
      llim = self.get_param_value("o1_hz")+self.get_param_value("sr_hz")+self.get_param_value("sw_hz")/2.0
      return llim-(ptval-1)*self.get_param_value("inc_hz")

  def sec2point(self, secval):
    """Converts Second value to point.
    Returns point index Value.
    """
    return secval/self.get_param("inc_tps").get_value()+1

  def point2sec(self, ptval):
    """Converts Second value to point.
    Returns point index Value.
    """
    return (ptval-1) * self.get_param("inc_tps").get_value()
  
  def hz2ppm(self, hzval):
    """Converts Hz value to PPM.
    Returns PPM Value.
    """
    if self.processed:
      return hzval/self.get_param_value("bf1")
  
  def ppm2hz(self, ppmval):
    """Converts PPM value to Hz.
    Returns Hz Value.
    """
    if self.processed:
      return ppmval*self.get_param_value("bf1")
  
  def old_units_to_new(self, old_units, new_units, values):
    """Change a tuple of values (or single value) form units to new units"""
    new_values=()
    
    if old_units==new_units:
      return values
    
    if (type(values) is not tuple) and (type(values) is not list):
      values=(values,)
    
    if old_units==NMRFriend.UNITS_PTS:
      if new_units==NMRFriend.UNITS_HZ:
        if self.processed:
          for value in values:
            new_values=new_values+(self.point2hz(value),)
        
      elif new_units==NMRFriend.UNITS_PPM:
        if self.processed:
          for value in values:
            new_values=new_values+(self.point2ppm(value),)
       
      elif new_units==NMRFriend.UNITS_SEC:
        for value in values:
          new_values=new_values+(self.point2sec(value),)
       
      
    elif old_units==NMRFriend.UNITS_HZ and self.processed:
      if new_units==NMRFriend.UNITS_PTS:
        for value in values:
          new_values=new_values+(self.hz2point(value),)
      elif new_units==NMRFriend.UNITS_PPM:
        for value in values:
          new_values=new_values+(self.hz2ppm(value),)
       
    elif old_units==NMRFriend.UNITS_PPM and self.processed:
      if new_units==NMRFriend.UNITS_HZ:
        for value in values:
          new_values=new_values+(self.ppm2hz(value),)
      elif new_units==NMRFriend.UNITS_PTS:
        for value in values:
          new_values=new_values+(self.ppm2point(value),)
       
    elif old_units==NMRFriend.UNITS_SEC:
      if new_units== NMRFriend.UNITS_PTS:
        for value in values:
          new_values=new_values+(self.sec2point(value),)

    if len(new_values)==1:
      return new_values[0]
    elif len(new_values)==0:
      values
    else:
      return new_values
  
  def set_spc(self, newReSpc, newImSpc):
    """Updates both Real and Imaginary spectra.
    Returns True if OK, False if not.
    """
    if len(newImSpc)!=len(newReSpc):
      print "Uncoherent Spectrum!"
      return False
    else:
      self.reSpc=newReSpc # Sets new Real Spectrum
      self.imSpc=newImSpc # Sets new Imaginary Spectrum
      self.set_param_value("si", len(self.reSpc)) # Update parameter
      return True
  
  def build_ph_spc(self):
    """Builds new phased spectra (Real & Imaginary) using phase corrections and pivot point.
    Returns True if OK, False if not.
    """
    
    # Checks if phcs and index are set
    if not self.paramset.param_isset("phc0", "phc1", "pivot"):
      print "Wrong initialisation of phase correction parameters"
      return False
    
    # Checks if lastphc are set
    if self.lastphc0==None:
      self.lastphc0=self.get_param_value("phc0")
      self.lastphc1=self.get_param_value("phc1")
      
    # Ok Here starts the fun
    newReSpc=[]
    newImSpc=[]
    for index in range(len(self.reSpc)):
      # Define difference between oldly used phc and newly used
      deltaphc0 = self.get_param_value("phc0") - self.lastphc0
      deltaphc1 = self.get_param_value("phc1") - self.lastphc1
      
      # Calculate value of theta following: theta[i]=phc0+phc1*(i-pivot)/nbpt of course this is in deg since phcs are in deg
      theta=(deltaphc0+deltaphc1*(index-self.get_param_value("pivot"))/self.get_param_value("nb_pt_spc"))*2*math.pi/360.0
      
      # Build Real Spectrum following: Re[i]=Re0[i]*cos(theta)-Im0[i]*sin(theta)
      newReSpc.append(self.reSpc[index]*math.cos(theta)-self.imSpc[index]*math.sin(theta));
      
      # Build Imaginary Spectrum following: Im[i]=Im0[i]*cos(theta)+Re0[i]*sin(theta)
      newImSpc.append(self.imSpc[index]*math.cos(theta)+self.reSpc[index]*math.sin(theta));
      
    # Updating values
    self.reSpc=newReSpc
    self.imSpc=newImSpc
    self.lastphc0=self.get_param_value("phc0")
    self.lastphc1=self.get_param_value("phc1")
    return True
  
  def set_phase(self, newphc0=None, newphc1=None):
    """Sets new value to phase correction parameters.
    Returns True if OK, False if not.
    """
    if (newphc0!=None):
      self.set_param_value("phc0", newphc0)
    if (newphc1!=None):
      self.set_param_value("phc1", newphc1)
    return self.build_ph_spc()

  def set_pivot(self, newPivot=None):
    """Set new value to pivot point.
    Returns True if OK, False if not.
    """
    if (newPivot!=None):
      self.set_param_value("pivot", newPivot)
    return self.build_ph_spc()
  
  def get_acqu_params(self):
    """Returns tuple containing acquisition parameters"""
    tmptup=()
    for param in self.paramset.get_params():
      if param.get_param_type() == NMRFriend.PARAM_TYPE_ACQU:
        tmptup+=(param,)
    return tmptup
  
  def get_proc_params(self):
    """Returns tuple containing processing parameters"""
    tmptup=()
    for param in self.paramset.get_params():
      if param.get_param_type() == NMRFriend.PARAM_TYPE_PROC:
        tmptup+=(param,)
    return tmptup
  
  def get_params_from_type(self, param_type):
    """Returns tuple containing parameters corresponding to param_type"""
    tmptup=()
    for param in self.paramset.get_params():
      if param.get_param_type() == param_type:
        tmptup+=(param,)
    return tmptup
  
  def get_params(self):
    """Returns tuple containing all parameters available."""
    return self.paramset.get_params()
  
  def get_params_names(self):
    """Returns tuple containing all parameters names."""
    return self.paramset.get_params_names()
  
  def get_param(self, name):
    """Returns NmrParam object corresponding to name given.
    Returns None if name is not a correct parameter name
    """
    return self.paramset.get_param(name)
  
  def __len__(self):
    return len(self.reSpc)

class SpcBruker(NmrSpc):
  """Class for Bruker NMR spectrum"""

  # list containing parameters
  # syntax for parameter:[name,desc,value,param_type,visibility,user_tweakable]
  list_param=NmrSpc.list_param[0:]
  list_param+=[["aqseq", "True if Aquisition is sequential", None, NMRFriend.PARAM_TYPE_ACQU, False, False], 
               ["bf1", "Resonance frequency of nucleus (MHz)", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
               ["bytorda", "Byte order for FID. True if Big Endian", None, NMRFriend.PARAM_TYPE_ACQU, False, False], 
               ["ns", "Number of scans", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
               ["nuc1", "Active nucleus", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
               ["nc", "Must be something", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
               ["o1", "Excitation frequency in Hz from Excitation reference SFO1", None, NMRFriend.PARAM_TYPE_ACQU, False, False], 
               ["overflw", "True if overflow occured", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
               ["sfo1", "Reference frequency for excitation", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
               ["te", "Temperature during acquisition", None, NMRFriend.PARAM_TYPE_ACQU, True, False], 
               ["bytordp", "Byte order for processed spectrum. True if Big Endian", None, NMRFriend.PARAM_TYPE_PROC, False, False], 
               ["ftsize", "Size of FT", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
               ["lb", "Line Broadening factor", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
               ["nc_proc", "Power of 2 factor used to display spectrum", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
               ["nsp", "Number of points used when shifting data points", None, NMRFriend.PARAM_TYPE_PROC, True, False], 
               ]
  
  def __init__(self, fid=[], spcRe=[], spcIm=[]):
    NmrSpc.__init__(self, fid, spcRe, spcIm)

class Limits(object):
    def __init__(self, spectrum):
        self.llim = int(len(spectrum) *  0.25)                      # left limit
        self.rlim = int(len(spectrum) *  0.75)     # right limit
        self.pivot = len(spectrum) / 2     # Pivot point
    
    def set_limits(self, limits):
        self.llim = min(limits)
        self.rlim = max(limits)
    
    def get_limits(self):
        return (self.llim, self.rlim)
    
    def set_pivot(self, pivot):
        self.pivot = pivot
    
    def get_pivot(self):
        return self.pivot
 
class Moments(object):
  """Handles NMR Spectrum Moments"""
  
  def __init__(self, spectrum):
    """Create Moments object.
    Calculate moments of spectrum
    """
    if(isinstance(spectrum, NmrSpc)):
      self.nmrspc= spectrum               # NMRSpc object used for moments calculation
      self.spc= spectrum.get_real_spc()   # Spectrum used for calculation
      self.M1zero=None
      
      self.moments_calculated=False
      self.m1=None
      self.m1sym=None
      self.m2=None
    
    else:
      print "Can't compute moments on object: This is not a NMR Spectrum!"
   
  def calc_m1_zero(self):
    """Calculate first moment zero."""
    # First Update spectrum
    self.update_spc()
    
    ar1=0.0
    ar2=0.0
    for i in range(self.llim, self.rlim):
      ar1+= self.spc[i]
      ar2+= self.spc[i]*i
    self.M1zero= int(round(ar2/ar1))
    
    return self.M1zero
  
  def get_m1_zero(self):
    """Return M1 zero"""
    return self.M1zero
  
  def calc_moments(self):
    """Moments calculation"""
    # First Update spectrum
    self.update_spc()
    
    if self.llim is None:
      self.find_spc_lim()
    if self.pivot is None:
      self.pivot=self.calc_m1_zero()
    else:
        self.calc_m1_zero()
    m0, m1, m1sym, m2=0.0, 0.0, 0.0, 0.0
    
    # perform baseline correction
    sum = 0.0
    spc = self.spc
    ncp = len(spc)
    llim = self.llim
    rlim = self.rlim
    
    for i in range(llim):
        sum += spc[i]
    
    for i in range(rlim-1,ncp):
        sum += spc[i]
    
    sums = sum/(ncp - rlim + llim + 1)
    
    for i in range(len(spc)):
        spc[i] = spc[i] - sums
    
    tmpM0, tmpM1sym, tmpM2=0.0, 0.0, 0.0
    
    # calculate moment by averaging 25 calculations
    for j in range(0, 50, 2):
      for i in range(self.llim+j, self.rlim-j):
        delta = i-self.pivot
        valspc = spc[i]
        toupai = 2*math.pi
        m0 += valspc
        m1 += valspc*delta*toupai
        m1sym += valspc*abs(delta)*toupai
        m2 += valspc*(delta*toupai)**2
      
        tmpM0 += valspc
        tmpM1sym += valspc*abs(delta)*toupai
        tmpM2 += valspc*(delta*toupai)**2
      
      tmpM2 = tmpM2/tmpM0
      tmpM1sym = tmpM1sym/tmpM0
      
      if j==0:
        m1symMin=tmpM1sym
        m1symMax=tmpM1sym
        m2Min=tmpM2
        m2Max=tmpM2
      else:
        if (tmpM1sym<m1symMin):
          m1symMin=tmpM1sym
        if (tmpM1sym>m1symMax):
          m1symMax=tmpM1sym
        if (tmpM2<m2Min):
          m2Min=tmpM2
        if (tmpM2>m2Max):
          m2Max=tmpM2
      
      tmpM0=0
      tmpM1sym=0
      tmpM2=0
    
    self.m2=m2/m0
    self.m1=m1/m0
    self.m1sym=m1sym/m0
    
#===============================================================================
#    print "################################"
#    print "Results of moments calculations:"
#    print "All results in points or points squared units!"
#    print "limits:", self.llim, "-", self.rlim
#    print "pivot:", self.pivot
#    print "M1 zero", self.calc_m1_zero()
#    print "M1 sym:", self.m1sym
#    print "M1:", self.m1
#    print "M2 :", self.m2
#===============================================================================

  def get_results_tupled(self):
    """Returns results in a list of tuples"""
    
    tup_res=[]
    
    # print limits
    tup_res.append(("title", "Limits used"))
    llim_pt, rlim_pt = self.nmrspc.limits.get_limits()
    tup_res.append(("Limits (pts)", str(llim_pt)+" to "+str(rlim_pt)))
    
    llim, rlim = self.nmrspc.old_units_to_new(NMRFriend.UNITS_PTS, NMRFriend.UNITS_HZ, (llim_pt, rlim_pt) )
    tup_res.append(("Limits (Hz)", str(int(min(rlim, llim)))+" to "+str(int(max(rlim, llim)))))
    
    llim, rlim=self.nmrspc.old_units_to_new(NMRFriend.UNITS_PTS, NMRFriend.UNITS_PPM, (llim_pt, rlim_pt))
    tup_res.append(("Limits (PPM)", str(int(min(rlim, llim)))+" to "+str(int(max(rlim, llim)))))
    
    # print pivot
    tup_res.append(("title", "Pivot point used"))
    pivot_pt = self.nmrspc.limits.get_pivot()
    tup_res.append(("Pivot (pts)", str(pivot_pt)))
    
    pivot=self.nmrspc.old_units_to_new(NMRFriend.UNITS_PTS, NMRFriend.UNITS_HZ, pivot_pt)
    tup_res.append(("Pivot (Hz)", str(pivot)))
    
    pivot=self.nmrspc.old_units_to_new(NMRFriend.UNITS_PTS, NMRFriend.UNITS_PPM, pivot_pt)
    tup_res.append(("Pivot (PPM)", str(pivot)))
    
    # print M1 zero
    tup_res.append(("title", "First order moment zero"))
    tup_res.append(("M1 zero (pts)", str(self.M1zero)))
    m1zero=self.nmrspc.old_units_to_new(NMRFriend.UNITS_PTS, NMRFriend.UNITS_HZ, self.M1zero)
    tup_res.append(("M1 zero (Hz)", str(m1zero)))
    m1zero=self.nmrspc.old_units_to_new(NMRFriend.UNITS_PTS, NMRFriend.UNITS_PPM, self.M1zero)
    tup_res.append(("M1 zero (PPM)", str(m1zero)))
    
    # print M1
    tup_res.append(("title", "First order moment"))
    tup_res.append(("M1 (pts)", str(self.m1)))
    m1=self.m1*abs(self.nmrspc.get_param_value("inc_hz"))
    tup_res.append(("M1 (Hz)", str(m1)))
    m1=self.m1*abs(self.nmrspc.get_param_value("inc_ppm"))
    tup_res.append(("M1 (PPM)", str(m1)))
    
    # print M1sym
    tup_res.append(("title", "First order moment (sym)"))
    tup_res.append(("M1sym (pts)", str(self.m1sym)))
    m1sym=self.m1sym*abs(self.nmrspc.get_param_value("inc_hz"))
    tup_res.append(("M1sym (Hz)", str(m1sym)))
    m1sym=self.m1sym*abs(self.nmrspc.get_param_value("inc_ppm"))
    tup_res.append(("M1sym (PPM)", str(m1sym)))
    
    # print M2
    tup_res.append(("title", "Second order moment"))
    tup_res.append(("M2 (pts^2)", str(self.m2)))
    m2=self.m2*abs(self.nmrspc.get_param_value("inc_hz"))**2
    tup_res.append(("M2 (Hz^2)", str(m2)))
    m2=self.m2*abs(self.nmrspc.get_param_value("inc_ppm"))**2
    tup_res.append(("M2 (PPM^2)", str(m2)))
    
    return tup_res

  def update_spc(self):
    """Update Spectrum value"""
    self.spc = self.nmrspc.get_real_spc()
    llim, rlim = self.nmrspc.limits.get_limits()
    pivot = self.nmrspc.limits.get_pivot()
    
    self.llim = int(llim)
    self.rlim = int(rlim)
    self.pivot = int(pivot)

class NmrParamSet:
  """Class for sets of NMR Parameters"""
  def __init__(self, listparam):
    """Create a parameter set and create parameters if given"""
    self.listparam={}
    
    for paramtocreate in listparam:
      if type(paramtocreate)==type([]) or len(paramtocreate)==6:
        self.listparam[paramtocreate[0]]=NmrParam(paramtocreate[0], paramtocreate[1], paramtocreate[2], paramtocreate[3], paramtocreate[4], paramtocreate[5])
      else:
        raise TypeError
      
  def param_exists(self, param_name):
    """Return True if exists a parameter with given name in parameter set.
    Otherwise, return False
    """
    if self.listparam.has_key(param_name):
      return True
    else:
      return False
    
  def get_param(self, name):
    """Returns NmrParam object corresponding to name given.
    Returns None if name is not a correct parameter name
    """
    if self.listparam.has_key(name):
      return self.listparam[name]
    else:
      return None
   
  def get_params(self):
    """Returns tuple containing all NmrParam object from NmrParamSet"""
    tmptup=()
    for param in self.listparam.values():
      tmptup+=(param,)
    return tmptup
  
  def add_param(self, newparam):
    """Add new parameter to parameter set.
    Parameter must be a NmrParam object or a list containing parameters to create a new NmrParam object.
    If parameter already exists in parameter set, add_param does nothing.
    """
    if type(newparam)==NmrParam(""):
      if self.listparam.has_key(newparam.get_name()):
        pass
      else:
        self.listparam[newparam.get_name()]=newparam
    elif type(newparam)==type([]) and len(newparam)==6:
      self.listparam[newparam[0]]=NmrParam(newparam[0], newparam[1], newparam[2], newparam[3], newparam[4], newparam[5])
    else:
      raise TypeError
    
  def remove_param(self, param_name):
    """Remove parameter which name is given from parameter set.
    Does nothing if parameter does not exist
    """
    if self.listparam.has_key(param_name):
      del self.listparam[param_name]
    else:
      pass
    
  def get_params_names(self):
    """Returns tuple containing parameter names"""
    temp=self.listparam.keys()
    temp.sort()
    return tuple(temp)

  def param_isset(self, *param_names):
    """Return True if all parameters given in argument are set (i.e. not equal to None).
    Return False if one of them is not set or not existing.
    """
    allOK=True
    for param_name in param_names:
      if not self.listparam.has_key(param_name):
        allOK=False
        break
      elif self.listparam[param_name].get_value()==None:
        allOK=False
        break
    return allOK
  